@@ -1,5 +1,4 @@
 """Functionality for autorenewal and associated juggling of configurations"""
-from __future__ import print_function
 
 import copy
 import itertools
@@ -8,28 +7,28 @@ import random
 import sys
 import time
 import traceback
+from typing import List
+from typing import Optional
 
 from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric import rsa
 from cryptography.hazmat.primitives.serialization import load_pem_private_key
-import OpenSSL
-import six
 import zope.component
 
-from acme.magic_typing import List
-from acme.magic_typing import Optional  # pylint: disable=unused-import
 from certbot import crypto_util
 from certbot import errors
 from certbot import interfaces
 from certbot import util
 from certbot._internal import cli
-from certbot._internal import client  # pylint: disable=unused-import
+from certbot._internal import client
 from certbot._internal import constants
 from certbot._internal import hooks
 from certbot._internal import storage
 from certbot._internal import updater
 from certbot._internal.plugins import disco as plugins_disco
 from certbot.compat import os
+from certbot.display import util as display_util
 
 logger = logging.getLogger(__name__)
 
@@ -40,7 +39,7 @@ logger = logging.getLogger(__name__)
 STR_CONFIG_ITEMS = ["config_dir", "logs_dir", "work_dir", "user_agent",
                     "server", "account", "authenticator", "installer",
                     "renew_hook", "pre_hook", "post_hook", "http01_address",
-                    "preferred_chain"]
+                    "preferred_chain", "key_type", "elliptic_curve"]
 INT_CONFIG_ITEMS = ["rsa_key_size", "http01_port"]
 BOOL_CONFIG_ITEMS = ["must_staple", "allow_subset_of_names", "reuse_key",
                      "autorenew"]
@@ -68,27 +67,28 @@ def _reconstitute(config, full_path):
     """
     try:
         renewal_candidate = storage.RenewableCert(full_path, config)
-    except (errors.CertStorageError, IOError):
-        logger.warning("", exc_info=True)
-        logger.warning("Renewal configuration file %s is broken. Skipping.", full_path)
+    except (errors.CertStorageError, IOError) as error:
+        logger.error("Renewal configuration file %s is broken.", full_path)
+        logger.error("The error was: %s\nSkipping.", str(error))
         logger.debug("Traceback was:\n%s", traceback.format_exc())
         return None
     if "renewalparams" not in renewal_candidate.configuration:
-        logger.warning("Renewal configuration file %s lacks "
+        logger.error("Renewal configuration file %s lacks "
                        "renewalparams. Skipping.", full_path)
         return None
     renewalparams = renewal_candidate.configuration["renewalparams"]
     if "authenticator" not in renewalparams:
-        logger.warning("Renewal configuration file %s does not specify "
+        logger.error("Renewal configuration file %s does not specify "
                        "an authenticator. Skipping.", full_path)
         return None
     # Now restore specific values along with their data types, if
     # those elements are present.
+    renewalparams = _remove_deprecated_config_elements(renewalparams)
     try:
         restore_required_config_elements(config, renewalparams)
         _restore_plugin_configs(config, renewalparams)
     except (ValueError, errors.Error) as error:
-        logger.warning(
+        logger.error(
             "An error occurred while parsing %s. The error was %s. "
             "Skipping the file.", full_path, str(error))
         logger.debug("Traceback was:\n%s", traceback.format_exc())
@@ -98,7 +98,7 @@ def _reconstitute(config, full_path):
         config.domains = [util.enforce_domain_sanity(d)
                           for d in renewal_candidate.names()]
     except errors.ConfigurationError as error:
-        logger.warning("Renewal configuration file %s references a cert "
+        logger.error("Renewal configuration file %s references a certificate "
                        "that contains an invalid domain name. The problem "
                        "was: %s. Skipping.", full_path, error)
         return None
@@ -118,7 +118,7 @@ def _restore_webroot_config(config, renewalparams):
     # see https://github.com/certbot/certbot/pull/7095
     if "webroot_path" in renewalparams and not cli.set_by_cli("webroot_path"):
         wp = renewalparams["webroot_path"]
-        if isinstance(wp, six.string_types):  # prior to 0.1.0, webroot_path was a string
+        if isinstance(wp, str):  # prior to 0.1.0, webroot_path was a string
             wp = [wp]
         config.webroot_path = wp
 
@@ -142,7 +142,7 @@ def _restore_plugin_configs(config, renewalparams):
     #      longer defined, stored copies of that parameter will be
     #      deserialized as strings by this logic even if they were
     #      originally meant to be some other type.
-    plugin_prefixes = []  # type: List[str]
+    plugin_prefixes: List[str] = []
     if renewalparams["authenticator"] == "webroot":
         _restore_webroot_config(config, renewalparams)
     else:
@@ -153,7 +153,7 @@ def _restore_plugin_configs(config, renewalparams):
 
     for plugin_prefix in set(plugin_prefixes):
         plugin_prefix = plugin_prefix.replace('-', '_')
-        for config_item, config_value in six.iteritems(renewalparams):
+        for config_item, config_value in renewalparams.items():
             if config_item.startswith(plugin_prefix + "_") and not cli.set_by_cli(config_item):
                 # Values None, True, and False need to be treated specially,
                 # As their types aren't handled correctly by configobj
@@ -178,15 +178,28 @@ def restore_required_config_elements(config, renewalparams):
 
     required_items = itertools.chain(
         (("pref_challs", _restore_pref_challs),),
-        six.moves.zip(BOOL_CONFIG_ITEMS, itertools.repeat(_restore_bool)),
-        six.moves.zip(INT_CONFIG_ITEMS, itertools.repeat(_restore_int)),
-        six.moves.zip(STR_CONFIG_ITEMS, itertools.repeat(_restore_str)))
+        zip(BOOL_CONFIG_ITEMS, itertools.repeat(_restore_bool)),
+        zip(INT_CONFIG_ITEMS, itertools.repeat(_restore_int)),
+        zip(STR_CONFIG_ITEMS, itertools.repeat(_restore_str)))
     for item_name, restore_func in required_items:
         if item_name in renewalparams and not cli.set_by_cli(item_name):
             value = restore_func(item_name, renewalparams[item_name])
             setattr(config, item_name, value)
 
 
+def _remove_deprecated_config_elements(renewalparams):
+    """Removes deprecated config options from the parsed renewalparams.
+
+    :param dict renewalparams: list of parsed renewalparams
+
+    :returns: list of renewalparams with deprecated config options removed
+    :rtype: dict
+
+    """
+    return {option_name: v for (option_name, v) in renewalparams.items()
+        if option_name not in cli.DEPRECATED_OPTIONS}
+
+
 def _restore_pref_challs(unused_name, value):
     """Restores preferred challenges from a renewal config file.
 
@@ -205,7 +218,7 @@ def _restore_pref_challs(unused_name, value):
     # If pref_challs has only one element, configobj saves the value
     # with a trailing comma so it's parsed as a list. If this comma is
     # removed by the user, the value is parsed as a str.
-    value = [value] if isinstance(value, six.string_types) else value
+    value = [value] if isinstance(value, str) else value
     return cli.parse_preferred_challenges(value)
 
 
@@ -276,34 +289,24 @@ def _restore_str(name, value):
 
 
 def should_renew(config, lineage):
-    "Return true if any of the circumstances for automatic renewal apply."
+    """Return true if any of the circumstances for automatic renewal apply."""
     if config.renew_by_default:
         logger.debug("Auto-renewal forced with --force-renewal...")
         return True
     if lineage.should_autorenew():
-        logger.info("Cert is due for renewal, auto-renewing...")
+        logger.info("Certificate is due for renewal, auto-renewing...")
         return True
     if config.dry_run:
-        logger.info("Cert not due for renewal, but simulating renewal for dry run")
+        logger.info("Certificate not due for renewal, but simulating renewal for dry run")
         return True
-    logger.info("Cert not yet due for renewal")
+    display_util.notify("Certificate not yet due for renewal")
     return False
 
 
 def _avoid_invalidating_lineage(config, lineage, original_server):
-    "Do not renew a valid cert with one from a staging server!"
-    # Some lineages may have begun with --staging, but then had production certs
-    # added to them
-    with open(lineage.cert) as the_file:
-        contents = the_file.read()
-    latest_cert = OpenSSL.crypto.load_certificate(
-        OpenSSL.crypto.FILETYPE_PEM, contents)
-    # all our test certs are from happy hacker fake CA, though maybe one day
-    # we should test more methodically
-    now_valid = "fake" not in repr(latest_cert.get_issuer()).lower()
-
+    """Do not renew a valid cert with one from a staging server!"""
     if util.is_staging(config.server):
-        if not util.is_staging(original_server) or now_valid:
+        if not util.is_staging(original_server):
             if not config.break_my_certs:
                 names = ", ".join(lineage.names())
                 raise errors.Error(
@@ -312,8 +315,8 @@ def _avoid_invalidating_lineage(config, lineage, original_server):
                     "unless you use the --break-my-certs flag!".format(names))
 
 
-def renew_cert(config, domains, le_client, lineage):
-    # type: (interfaces.IConfig, Optional[List[str]], client.Client, storage.RenewableCert) -> None
+def renew_cert(config: interfaces.IConfig, domains: Optional[List[str]], le_client: client.Client,
+               lineage: storage.RenewableCert) -> None:
     """Renew a certificate lineage."""
     renewal_params = lineage.configuration["renewalparams"]
     original_server = renewal_params.get("server", cli.flag_default("server"))
@@ -340,47 +343,49 @@ def renew_cert(config, domains, le_client, lineage):
 
 
 def report(msgs, category):
-    "Format a results report for a category of renewal outcomes"
+    """Format a results report for a category of renewal outcomes"""
     lines = ("%s (%s)" % (m, category) for m in msgs)
     return "  " + "\n  ".join(lines)
 
 
-def _renew_describe_results(config, renew_successes, renew_failures,
-                            renew_skipped, parse_failures):
+def _renew_describe_results(config: interfaces.IConfig, renew_successes: List[str],
+                            renew_failures: List[str], renew_skipped: List[str],
+                            parse_failures: List[str]) -> None:
+    """
+    Print a report to the terminal about the results of the renewal process.
 
-    out = []  # type: List[str]
-    notify = out.append
-    disp = zope.component.getUtility(interfaces.IDisplay)
+    :param interfaces.IConfig config: Configuration
+    :param list renew_successes: list of fullchain paths which were renewed
+    :param list renew_failures: list of fullchain paths which failed to be renewed
+    :param list renew_skipped: list of messages to print about skipped certificates
+    :param list parse_failures: list of renewal parameter paths which had erorrs
+    """
+    notify = display_util.notify
+    notify_error = logger.error
 
-    def notify_error(err):
-        """Notify and log errors."""
-        notify(str(err))
-        logger.error(err)
+    notify('\n{}'.format(display_util.SIDE_FRAME))
+
+    renewal_noun = "simulated renewal" if config.dry_run else "renewal"
 
-    if config.dry_run:
-        notify("** DRY RUN: simulating 'certbot renew' close to cert expiry")
-        notify("**          (The test certificates below have not been saved.)")
-    notify("")
     if renew_skipped:
-        notify("The following certs are not due for renewal yet:")
+        notify("The following certificates are not due for renewal yet:")
         notify(report(renew_skipped, "skipped"))
     if not renew_successes and not renew_failures:
-        notify("No renewals were attempted.")
+        notify("No {renewal}s were attempted.".format(renewal=renewal_noun))
         if (config.pre_hook is not None or
                 config.renew_hook is not None or config.post_hook is not None):
             notify("No hooks were run.")
     elif renew_successes and not renew_failures:
-        notify("Congratulations, all renewals succeeded. The following certs "
-               "have been renewed:")
+        notify("Congratulations, all {renewal}s succeeded: ".format(renewal=renewal_noun))
         notify(report(renew_successes, "success"))
     elif renew_failures and not renew_successes:
-        notify_error("All renewal attempts failed. The following certs could "
-               "not be renewed:")
+        notify_error("All %ss failed. The following certificates could "
+               "not be renewed:", renewal_noun)
         notify_error(report(renew_failures, "failure"))
     elif renew_failures and renew_successes:
-        notify("The following certs were successfully renewed:")
+        notify("The following {renewal}s succeeded:".format(renewal=renewal_noun))
         notify(report(renew_successes, "success") + "\n")
-        notify_error("The following certs could not be renewed:")
+        notify_error("The following %ss failed:", renewal_noun)
         notify_error(report(renew_failures, "failure"))
 
     if parse_failures:
@@ -388,11 +393,7 @@ def _renew_describe_results(config, renew_successes, renew_failures,
                "were invalid: ")
         notify(report(parse_failures, "parsefail"))
 
-    if config.dry_run:
-        notify("** DRY RUN: simulating 'certbot renew' close to cert expiry")
-        notify("**          (The test certificates above have not been saved.)")
-
-    disp.notification("\n".join(out), wrap=False)
+    notify(display_util.SIDE_FRAME)
 
 
 def handle_renewal_request(config):
@@ -438,7 +439,7 @@ def handle_renewal_request(config):
         try:
             renewal_candidate = _reconstitute(lineage_config, renewal_file)
         except Exception as e:  # pylint: disable=broad-except
-            logger.warning("Renewal configuration file %s (cert: %s) "
+            logger.error("Renewal configuration file %s (cert: %s) "
                            "produced an unexpected error: %s. Skipping.",
                            renewal_file, lineagename, e)
             logger.debug("Traceback was:\n%s", traceback.format_exc())
@@ -449,7 +450,8 @@ def handle_renewal_request(config):
             if renewal_candidate is None:
                 parse_failures.append(renewal_file)
             else:
-                # XXX: ensure that each call here replaces the previous one
+                # This call is done only for retro-compatibility purposes.
+                # TODO: Remove this call once zope dependencies are removed from Certbot.
                 zope.component.provideUtility(lineage_config)
                 renewal_candidate.ensure_deployed()
                 from certbot._internal import main
@@ -482,9 +484,10 @@ def handle_renewal_request(config):
 
         except Exception as e:  # pylint: disable=broad-except
             # obtain_cert (presumably) encountered an unanticipated problem.
-            logger.warning("Attempting to renew cert (%s) from %s produced an "
-                           "unexpected error: %s. Skipping.", lineagename,
-                               renewal_file, e)
+            logger.error(
+                "Failed to renew certificate %s with error: %s",
+                lineagename, e
+            )
             logger.debug("Traceback was:\n%s", traceback.format_exc())
             renew_failures.append(renewal_candidate.fullchain)
 
@@ -501,11 +504,14 @@ def handle_renewal_request(config):
     logger.debug("no renewal failures")
 
 
-def _update_renewal_params_from_key(key_path, config):
-    # type: (str, interfaces.IConfig) -> None
+def _update_renewal_params_from_key(key_path: str, config: interfaces.IConfig) -> None:
     with open(key_path, 'rb') as file_h:
         key = load_pem_private_key(file_h.read(), password=None, backend=default_backend())
     if isinstance(key, rsa.RSAPrivateKey):
+        config.key_type = 'rsa'
         config.rsa_key_size = key.key_size
+    elif isinstance(key, ec.EllipticCurvePrivateKey):
+        config.key_type = 'ecdsa'
+        config.elliptic_curve = key.curve.name
     else:
         raise errors.Error('Key at {0} is of an unsupported type: {1}.'.format(key_path, type(key)))
