@@ -18,27 +18,27 @@ Note, that all annotated challenges act as a proxy objects::
 
 """
 import logging
+from typing import Type
 
 import josepy as jose
 
 from acme import challenges
-
+from acme.challenges import Challenge
 
 logger = logging.getLogger(__name__)
 
 
-
 class AnnotatedChallenge(jose.ImmutableMap):
     """Client annotated challenge.
 
     Wraps around server provided challenge and annotates with data
     useful for the client.
 
-    :ivar challb: Wrapped `~.ChallengeBody`.
+    :ivar ~.challb: Wrapped `~.ChallengeBody`.
 
     """
     __slots__ = ('challb',)
-    acme_type = NotImplemented
+    _acme_type: Type[Challenge] = NotImplemented
 
     def __getattr__(self, name):
         return getattr(self.challb, name)
