@@ -2,16 +2,15 @@
 
 import abc
 import logging
-import stat
 from time import sleep
 
 import configobj
 import zope.interface
 
 from acme import challenges
-
 from certbot import errors
 from certbot import interfaces
+from certbot.compat import filesystem
 from certbot.compat import os
 from certbot.display import ops
 from certbot.display import util as display_util
@@ -26,7 +25,7 @@ class DNSAuthenticator(common.Plugin):
     """Base class for DNS  Authenticators"""
 
     def __init__(self, config, name):
-        super(DNSAuthenticator, self).__init__(config, name)
+        super().__init__(config, name)
 
         self._attempt_cleanup = False
 
@@ -38,13 +37,25 @@ class DNSAuthenticator(common.Plugin):
             help='The number of seconds to wait for DNS to propagate before asking the ACME server '
                  'to verify the DNS record.')
 
-    def get_chall_pref(self, unused_domain):  # pylint: disable=missing-docstring,no-self-use
+    def auth_hint(self, failed_achalls):
+        delay = self.conf('propagation-seconds')
+        return (
+            'The Certificate Authority failed to verify the DNS TXT records created by --{name}. '
+            'Ensure the above domains are hosted by this DNS provider, or try increasing '
+            '--{name}-propagation-seconds (currently {secs} second{suffix}).'
+            .format(name=self.name, secs=delay, suffix='s' if delay != 1 else '')
+        )
+
+    def get_chall_pref(self, unused_domain):  # pylint: disable=missing-function-docstring
         return [challenges.DNS01]
 
-    def prepare(self): # pylint: disable=missing-docstring
+    def prepare(self): # pylint: disable=missing-function-docstring
         pass
 
-    def perform(self, achalls): # pylint: disable=missing-docstring
+    def more_info(self) -> str:  # pylint: disable=missing-function-docstring
+        raise NotImplementedError()
+
+    def perform(self, achalls): # pylint: disable=missing-function-docstring
         self._setup_credentials()
 
         self._attempt_cleanup = True
@@ -61,13 +72,13 @@ class DNSAuthenticator(common.Plugin):
         # DNS updates take time to propagate and checking to see if the update has occurred is not
         # reliable (the machine this code is running on might be able to see an update before
         # the ACME server). So: we sleep for a short amount of time we believe to be long enough.
-        logger.info("Waiting %d seconds for DNS changes to propagate",
+        display_util.notify("Waiting %d seconds for DNS changes to propagate" %
                     self.conf('propagation-seconds'))
         sleep(self.conf('propagation-seconds'))
 
         return responses
 
-    def cleanup(self, achalls):  # pylint: disable=missing-docstring
+    def cleanup(self, achalls):  # pylint: disable=missing-function-docstring
         if self._attempt_cleanup:
             for achall in achalls:
                 domain = achall.domain
@@ -140,7 +151,8 @@ class DNSAuthenticator(common.Plugin):
 
             setattr(self.config, self.dest(key), os.path.abspath(os.path.expanduser(new_value)))
 
-    def _configure_credentials(self, key, label, required_variables=None, validator=None):
+    def _configure_credentials(self, key, label, required_variables=None,
+                               validator=None) -> 'CredentialsConfiguration':
         """
         As `_configure_file`, but for a credential configuration file.
 
@@ -198,8 +210,7 @@ class DNSAuthenticator(common.Plugin):
 
         if code == display_util.OK:
             return response
-        else:
-            raise errors.PluginError('{0} required to proceed.'.format(label))
+        raise errors.PluginError('{0} required to proceed.'.format(label))
 
     @staticmethod
     def _prompt_for_file(label, validator=None):
@@ -232,11 +243,10 @@ class DNSAuthenticator(common.Plugin):
 
         if code == display_util.OK:
             return response
-        else:
-            raise errors.PluginError('{0} required to proceed.'.format(label))
+        raise errors.PluginError('{0} required to proceed.'.format(label))
 
 
-class CredentialsConfiguration(object):
+class CredentialsConfiguration:
     """Represents a user-supplied filed which stores API credentials."""
 
     def __init__(self, filename, mapper=lambda x: x):
@@ -303,8 +313,8 @@ def validate_file(filename):
     if not os.path.exists(filename):
         raise errors.PluginError('File not found: {0}'.format(filename))
 
-    if not os.path.isfile(filename):
-        raise errors.PluginError('Path is not a file: {0}'.format(filename))
+    if os.path.isdir(filename):
+        raise errors.PluginError('Path is a directory: {0}'.format(filename))
 
 
 def validate_file_permissions(filename):
@@ -312,8 +322,7 @@ def validate_file_permissions(filename):
 
     validate_file(filename)
 
-    permissions = stat.S_IMODE(os.stat(filename).st_mode)
-    if permissions & stat.S_IRWXO:
+    if filesystem.has_world_permissions(filename):
         logger.warning('Unsafe permissions on credentials configuration file: %s', filename)
 
 
